==> pb_common-0.75.tar.gz: pb_common-0.75/pb_common/context.py <==
import json
from urllib.parse import urlparse

from pb_common.config.configure import ConfigCreator
from pb_common.db import AbstractDbProvider
from pb_common.db.ad_provider import LdapProvider
from pb_common.db.oracle_provider import OracleProvider
from pb_common.db.redis_provider import RedisProvider
from pb_common.services.bender_provider import ClientProvider, Client
from pb_common.services.dbo_provider import Dbo2ServiceProvider
from pb_common.services.fimi_provider import FimiProvider
from pb_common.test_data import TestDataKeeper
from pb_common.utils.logger import get_logger
from pb_common.utils.pytest_extension import attach, step

logger = get_logger(__name__)


class UnknownDbException(Exception):
    pass


class UnknownServiceException(Exception):
    pass


class UnknownConfigException(Exception):
    pass


class BaseContext(object):
    def __init__(self, config):
        self._env = ConfigCreator(config).reload()
        self._services = dict()
        self._db_registry = dict()
        self._path = []
        logger.debug('Context {} is running!'.format(self))

    @property
    def config(self):
        return self._env

    def _databases_close(self):
        for name, database in self._db_registry.items():
            if isinstance(database, AbstractDbProvider):
                database.close()

    def close(self):
        """
        Закрыть соединения с БД
        :return:
        """
        self._databases_close()
        logger.debug("Shutdown context!")

    def _get_item_in_config(self, *path, name):
        """
        Получить объект из конфига
        :param path: путь до объекта
        :param name: ключ объекта
        :return: объект конфига
        """
        env = self._env
        for item in path:
            env = getattr(env, item, None)
            if not env:
                raise UnknownConfigException('Не найден конфиг: {}'.format(item))
        return getattr(env, name, None)

    def get_service(self, *path, name, provider, **kwargs):
        """
        Получить провайдер для работы с сервисом
        :param path: путь до сервиса в конфиге
        :param name: имя сервиса
        :param provider: провайдер для работы с сервисом
        :return: инстанс провайдера для работы с сервисом
        """
        if name not in self._services:
            env = self._get_item_in_config(*path, name=name)
            if not env:
                raise UnknownServiceException('Не найден конфиг сервиса: {}'.format(name))
            if kwargs:
                env.update(kwargs)
            self._services.update({name: provider(service='{} Service'.format(name.title()), **dict(env))})
        return self._services[name]

    def get_db_provider(self, *path, name, provider, query_storage=None):
        """
        Получить провайдер для работы с БД
        :param path: путь до БД в конфиге
        :param name: имя БД
        :param provider: провайдер для работы с БД
        :param query_storage: хранилище запросов к БД
        :return: объект для работы с БД
        """
        if name not in self._db_registry:
            env = self._get_item_in_config(*path, name=name)
            if not env:
                raise UnknownDbException('Не найден конфиг БД: {}'.format(name))
            self._db_registry.update({name: provider(**dict(env)).connect()})
            if query_storage:
                self._db_registry[name].query_storage = query_storage
        return self._db_registry[name]


class Context(BaseContext):
    @property
    def client_service(self):
        """
        Провайдер для запросов в ClientService
        """
        return self.get_service('services', 'dbo', name='client', provider=Dbo2ServiceProvider)

    @property
    def card_service(self):
        """
        Провайдер для запросов в CardService
        """
        return self.get_service('services', 'dbo', name='card', provider=Dbo2ServiceProvider)

    @property
    def session_service(self):
        """
        Провайдер для запросов в SessionService
        """
        return self.get_service('services', 'dbo', name='session', provider=Dbo2ServiceProvider)

    @property
    def registration_service(self):
        """
        Провайдер для запросов в RegistrationService
        """
        ib_url = self.config.get('baseUrls', {}).get('ib')
        if ib_url:
            url = urlparse(ib_url)
            if url.port:
                return self.get_service(
                    'services', 'dbo', name='registration', provider=Dbo2ServiceProvider, port=url.port)
        return self.get_service('services', 'dbo', name='registration', provider=Dbo2ServiceProvider)

    @property
    def contract_service(self):
        """
        Провайдер для запросов в ContractService
        """
        return self.get_service('services', 'dbo', name='contract', provider=Dbo2ServiceProvider)

    @property
    def token_provider_service(self):
        """
        Провайдер для запросов в TokenProvider
        """
        return self.get_service('services', 'dbo', name='tokenprovider', provider=Dbo2ServiceProvider)

    @property
    def bender_hub(self):
        """
        Провайдер для запросов в Bender (получение тестовых пользователей)
        """
        return self.get_service('services', name='benderhub', provider=Dbo2ServiceProvider)

    @property
    def fimi(self):
        """
        Провайдер для запросов в Мультикарту
        """
        return self.get_service('services', 'esb', name='fimi', provider=FimiProvider)

    @property
    def redis_provider(self):
        """
        Провайдер для запросов в Redis
        """
        if not self._db_registry.get('redis'):
            env = self._env.db.redis
            env.hosts = list(map(lambda x: x if isinstance(x, tuple) else tuple(x.split(':')), env.hosts))
            self._db_registry.update({'redis': RedisProvider(**dict(env)).connect()})
        return self._db_registry['redis']

    @property
    def ldap_provider(self):
        """
        Провайдер для запросов в LDAP (хранится авторизацонные данные клиентов)
        """
        return self.get_db_provider('db', name='ldap', provider=LdapProvider)

    @property
    def auth_provider(self):
        """
        Провайдер для работы с БД Oracle -  AUTH (хранится авторизацонные данные клиентов)
        """
        return self.get_db_provider('db', name='auth', provider=OracleProvider)

    @property
    def vabank_provider(self):
        """
        Провайдер для работы с БД Oracle -  VABANK (договоры клиентов)
        """
        return self.get_db_provider('db', name='vabank', provider=OracleProvider)

    @property
    def siebel_provider(self):
        """
        Провайдер для работы с БД Oracle - Siebel (информация о клиенте, заявках и договорах)
        """
        return self.get_db_provider('db', name='siebel', provider=OracleProvider)

    @property
    def cdr_provider(self):
        """
        Провайдер для работы с БД Oracle - CDR
        """
        return self.get_db_provider('db', name='cdr', provider=OracleProvider)

    @property
    def sas_provider(self):
        """
        Провайдер для работы с БД Oracle - SAS
        """
        return self.get_db_provider('db', name='sas', provider=OracleProvider)


class PyTestContext(Context):
    """
    Контекст для py.test - тестов
    """
    def __init__(self, config, test_data_keeper=None):
        """
        Инициализация контекста
        :param config: конфиг
        :param test_data_keeper: объект-хранитель тестовых данных
        """
        super().__init__(config)
        self.client_provider = ClientProvider(self)
        if not test_data_keeper:
            test_data_keeper = TestDataKeeper()
        self.test_data = test_data_keeper

    @property
    def client(self):
        """
        Получить данные клиента из хранителя тестовых данных
        :return: Client
        """
        return self.test_data.client

    @client.setter
    def client(self, value: Client):
        """
        Записать данные клиента в хранителя тестовых данных
        :param value: Client
        """
        if isinstance(value, Client):
            self.test_data.client = value
        else:
            raise AttributeError('Ожидается объект Client')

    @client.deleter
    def client(self):
        """
        Удалить клиента из тестовых данных
        """
        del self.test_data.client

    @step('Получить клиента для теста: task_id={task_id}, alias={alias}')
    def get_client(self, task_id=None, alias=None):
        """
        Получить клиента из хранилища тестовых пользователей
        :param alias: alias клиента
        :param task_id: task_id клиента
        :return: MobileContext
        """
        self.client = self.client_provider('GET', task_id, alias)
        attach('Пользователь', json.dumps(self.client_provider.user, indent=4, ensure_ascii=False))
        return self

    @step('Подготовить нового клиента для теста')
    def create_client(self, case_id: int, timeout: int = 240, **kwargs):
        """
        Подготовить нового клиента в хранилище тестовых пользователей
        :param timeout: время ожидания подготовки клиента в секундах
        :param case_id: идентификатор кейса
        :param kwargs: здесь можно указать парметры запуска задачи
                         -  params=List[Dict(key='', value=''),..., Dict(key='', value='')];
                       если эти данные можно переиспользовать в автотестах
                         -  auto_test=Dict[alias: str, parallel_use: bool]
        """
        self.client = self.client_provider('CREATE_TASK', case_id=case_id, is_client=True, timeout=timeout, **kwargs)
        attach('Пользователь', json.dumps(self.client_provider.user, indent=4, ensure_ascii=False))
        return self

    @step('Получить клиента для теста')
    def get_client_custom(self, task_id: str):
        """
        Получить клиента из Бендера
        :param task_id: идентификатор задачи
        """
        user = self.client_provider('GET_TASK', task_id=task_id)
        self.client = Client(self, **user['response'])
        attach('Пользователь', json.dumps(user, indent=4, ensure_ascii=False))
        return self

    @step('Подготовить тестовые данные')
    def create_test_data(self, case_id: int, timeout: int = 240, **kwargs):
        """
        #TODO: нужен ли этот метод? Нужно ли куда-то записывать test_data
        Подготовить тестовые данные через Бендера
        :param timeout: время ожидания подготовки клиента в секундах
        :param case_id: идентификатор кейса
        :param kwargs: здесь можно указать парметры запуска задачи
                         -  params=List[Dict(key='', value=''),..., Dict(key='', value='')];
                       если эти данные можно переиспользовать в автотестах
                         -  auto_test=Dict[alias: str, parallel_use: bool]
        """
        test_data = self.client_provider('CREATE_TASK', case_id=case_id, is_client=False, timeout=timeout, **kwargs)
        attach('Тестовые данные', json.dumps(test_data, indent=4, ensure_ascii=False))
        return self

    @step('Освободить клиента после теста')
    def release_client(self):
        """
        Отправить запрос в хранилище тестовых пользователей на освобождение клиента
        :return: MobileContext
        """
        if self.client:
            self.client_provider('RELEASE')
            del self.client
        return self

    @step('Запустить подготовку клиента после теста')
    def re_create_client(self):
        """
        Отправить запрос в хранилище тестовых пользователей на запуск подготовки такого же клиента
        :return: MobileContext
        """
        if self.client:
            self.client_provider('RERUN')
        return self

    @step('Удалить клиента после теста')
    def delete_client(self):
        """
        Отправить запрос в хранилище тестовых пользователей - пометить клиента удаленным
        :return: MobileContext
        """
        if self.client:
            self.client_provider('DELETE')
            del self.client
        return self

==> pb_common-0.96.tar.gz: pb_common-0.96/pb_common/context.py <==
import json
from urllib.parse import urlparse

from pb_common.config.configure import ConfigCreator
from pb_common.db import AbstractDbProvider
from pb_common.db.ad_provider import LdapProvider
from pb_common.db.oracle_provider import OracleProvider
from pb_common.db.redis_provider import RedisProvider
from pb_common.services.bender_provider import ClientProvider, Client
from pb_common.services.dbo_provider import Dbo2ServiceProvider
from pb_common.services.fimi_provider import FimiProvider
from pb_common.test_data import TestDataKeeper
from pb_common.utils.logger import get_logger
from pb_common.utils.pytest_extension import attach, step

logger = get_logger(__name__)


class UnknownDbException(Exception):
    pass


class UnknownServiceException(Exception):
    pass


class UnknownConfigException(Exception):
    pass


class BaseContext(object):
    def __init__(self, config):
        self._env = ConfigCreator(config).reload()
        self._services = dict()
        self._db_registry = dict()
        self._path = []
        logger.debug('Context {} is running!'.format(self))

    @property
    def config(self):
        """Окружение"""
        return self._env

    @property
    def env(self):
        """
        Получить название окружения из конфига ключ envName или название файла конфига
        :return:
        """
        try:
            return self.config.__getitem__('envName')
        except KeyError:
            return self.config.__env__

    def _databases_close(self):
        for name, database in self._db_registry.items():
            if isinstance(database, AbstractDbProvider):
                database.close()

    def close(self):
        """
        Закрыть соединения с БД
        :return:
        """
        self._databases_close()
        logger.debug("Shutdown context!")

    def _get_item_in_config(self, *path, name):
        """
        Получить объект из конфига
        :param path: путь до объекта
        :param name: ключ объекта
        :return: объект конфига
        """
        env = self._env
        for item in path:
            env = getattr(env, item, None)
            if not env:
                raise UnknownConfigException('Не найден конфиг: {}'.format(item))
        return getattr(env, name, None)

    def get_service(self, *path, name, provider, **kwargs):
        """
        Получить провайдер для работы с сервисом
        :param path: путь до сервиса в конфиге
        :param name: имя сервиса
        :param provider: провайдер для работы с сервисом
        :return: инстанс провайдера для работы с сервисом
        """
        if name not in self._services:
            env = self._get_item_in_config(*path, name=name)
            if not env:
                raise UnknownServiceException('Не найден конфиг сервиса: {}'.format(name))
            if kwargs:
                env.update(kwargs)
            self._services.update({name: provider(service='{} Service'.format(name.title()), **dict(env))})
        return self._services[name]

    def get_db_provider(self, *path, name, provider, query_storage=None):
        """
        Получить провайдер для работы с БД
        :param path: путь до БД в конфиге
        :param name: имя БД
        :param provider: провайдер для работы с БД
        :param query_storage: хранилище запросов к БД
        :return: объект для работы с БД
        """
        if name not in self._db_registry:
            env = self._get_item_in_config(*path, name=name)
            if not env:
                raise UnknownDbException('Не найден конфиг БД: {}'.format(name))
            self._db_registry.update({name: provider(**dict(env)).connect()})
            if query_storage:
                self._db_registry[name].query_storage = query_storage
        return self._db_registry[name]


class Context(BaseContext):
    @property
    def client_service(self):
        """
        Провайдер для запросов в ClientService
        """
        return self.get_service('services', 'dbo', name='client', provider=Dbo2ServiceProvider)

    @property
    def card_service(self):
        """
        Провайдер для запросов в CardService
        """
        return self.get_service('services', 'dbo', name='card', provider=Dbo2ServiceProvider)

    @property
    def session_service(self):
        """
        Провайдер для запросов в SessionService
        """
        return self.get_service('services', 'dbo', name='session', provider=Dbo2ServiceProvider)

    @property
    def registration_service(self):
        """
        Провайдер для запросов в RegistrationService
        """
        ib_url = self.config.get('baseUrls', {}).get('ib')
        if ib_url:
            url = urlparse(ib_url)
            if url.port:
                return self.get_service(
                    'services', 'dbo', name='registration', provider=Dbo2ServiceProvider, port=url.port)
        return self.get_service('services', 'dbo', name='registration', provider=Dbo2ServiceProvider)

    @property
    def contract_service(self):
        """
        Провайдер для запросов в ContractService
        """
        return self.get_service('services', 'dbo', name='contract', provider=Dbo2ServiceProvider)

    @property
    def dashboard_service(self):
        """
        Провайдер для запросов в DashboardService
        """
        return self.get_service('services', 'dbo', name='dashboard', provider=Dbo2ServiceProvider)

    @property
    def menu_service(self):
        """
        Провайдер для запросов в DboNet Menu сервис
        """
        return self.get_service('services', 'dbo', name='menu', provider=Dbo2ServiceProvider)

    @property
    def token_provider_service(self):
        """
        Провайдер для запросов в TokenProvider
        """
        return self.get_service('services', 'dbo', name='tokenprovider', provider=Dbo2ServiceProvider)

    @property
    def bender_hub(self):
        """
        Провайдер для запросов в Bender (получение тестовых пользователей)
        """
        return self.get_service('services', name='benderhub', provider=Dbo2ServiceProvider)

    @property
    def fimi(self):
        """
        Провайдер для запросов в Мультикарту
        """
        return self.get_service('services', 'esb', name='fimi', provider=FimiProvider)

    @property
    def redis_provider(self):
        """
        Провайдер для запросов в Redis
        """
        if not self._db_registry.get('redis'):
            env = self._env.db.redis
            env.hosts = list(map(lambda x: x if isinstance(x, tuple) else tuple(x.split(':')), env.hosts))
            self._db_registry.update({'redis': RedisProvider(**dict(env)).connect()})
        return self._db_registry['redis']

    @property
    def ldap_provider(self):
        """
        Провайдер для запросов в LDAP (хранится авторизацонные данные клиентов)
        """
        return self.get_db_provider('db', name='ldap', provider=LdapProvider)

    @property
    def auth_provider(self):
        """
        Провайдер для работы с БД Oracle -  AUTH (хранится авторизацонные данные клиентов)
        """
        return self.get_db_provider('db', name='auth', provider=OracleProvider)

    @property
    def vabank_provider(self):
        """
        Провайдер для работы с БД Oracle -  VABANK (договоры клиентов)
        """
        return self.get_db_provider('db', name='vabank', provider=OracleProvider)

    @property
    def siebel_provider(self):
        """
        Провайдер для работы с БД Oracle - Siebel (информация о клиенте, заявках и договорах)
        """
        return self.get_db_provider('db', name='siebel', provider=OracleProvider)

    @property
    def cdr_provider(self):
        """
        Провайдер для работы с БД Oracle - CDR
        """
        return self.get_db_provider('db', name='cdr', provider=OracleProvider)

    @property
    def jkh_provider(self):
        """
        Провайдер для работы с БД Oracle - CDR JKH
        """
        return self.get_db_provider('db', name='jkh', provider=OracleProvider)

    @property
    def sas_provider(self):
        """
        Провайдер для работы с БД Oracle - SAS
        """
        return self.get_db_provider('db', name='sas', provider=OracleProvider)


class PyTestContext(Context):
    """
    Контекст для py.test - тестов
    """
    def __init__(self, config, test_data_keeper=None):
        """
        Инициализация контекста
        :param config: конфиг
        :param test_data_keeper: объект-хранитель тестовых данных
        """
        super().__init__(config)
        self.client_provider = ClientProvider(self)
        if not test_data_keeper:
            test_data_keeper = TestDataKeeper()
        self.test_data = test_data_keeper

    @property
    def client(self):
        """
        Получить данные клиента из хранителя тестовых данных
        :return: Client
        """
        return self.test_data.client

    @client.setter
    def client(self, value: Client):
        """
        Записать данные клиента в хранителя тестовых данных
        :param value: Client
        """
        if isinstance(value, Client):
            self.test_data.client = value
        else:
            raise AttributeError('Ожидается объект Client')

    @client.deleter
    def client(self):
        """
        Удалить клиента из тестовых данных
        """
        del self.test_data.client

    @step('Получить клиента для теста: task_id={task_id}, alias={alias}')
    def get_client(self, task_id=None, alias=None):
        """
        Получить клиента из хранилища тестовых пользователей
        :param alias: alias клиента
        :param task_id: task_id клиента
        :return: MobileContext
        """
        self.client = self.client_provider('GET', task_id, alias)
        attach('Пользователь', json.dumps(self.client_provider.user, indent=4, ensure_ascii=False))
        return self

    @step('Подготовить нового клиента для теста')
    def create_client(self, case_id: int, timeout: int = 240, **kwargs):
        """
        Подготовить нового клиента в хранилище тестовых пользователей
        :param timeout: время ожидания подготовки клиента в секундах
        :param case_id: идентификатор кейса
        :param kwargs: здесь можно указать параметры запуска задачи
                  -  params=List[Dict(key='ключ_1', value='значение_1'),..., Dict(key='ключ_N', value='значение_N')];
                если эти данные можно переиспользовать в автотестах
                  -  auto_test=Dict[alias: str, parallel_use: bool]
        """
        self.client = self.client_provider('CREATE_TASK', case_id=case_id, is_client=True, timeout=timeout, **kwargs)
        attach('Пользователь', json.dumps(self.client_provider.user, indent=4, ensure_ascii=False))
        return self

    @step('Получить клиента для теста')
    def get_client_custom(self, task_id: str):
        """
        Получить клиента из Бендера
        :param task_id: идентификатор задачи
        """
        user = self.client_provider('GET_TASK', task_id=task_id)
        self.client = Client(self, **user['response'])
        attach('Пользователь', json.dumps(user, indent=4, ensure_ascii=False))
        return self

    @step('Получить вспомогательного клиента для теста')
    def get_client_info(self, task_id=None, alias=None, test_data_key='client_info'):
        """
        Метод для получения данных по клиенту из хранилища тестовых пользователей и записи этих данных
        в context по ключу
        :param alias: alias клиента
        :param task_id: task_id клиента
        :param test_data_key: ключ, по которому нужно записать данные клиента. По умолчанию запись в 'client_info'
        :return: client_info
        """
        client_info = self.client_provider('GET_TEST_CLIENT', task_id, alias)
        setattr(self.test_data, test_data_key, client_info)
        return client_info

    @step('Подготовить тестовые данные')
    def create_test_data(self, case_id: int, timeout: int = 240, **kwargs):
        """
        #TODO: нужен ли этот метод? Нужно ли куда-то записывать test_data
        Подготовить тестовые данные через Бендера
        :param timeout: время ожидания подготовки клиента в секундах
        :param case_id: идентификатор кейса
        :param kwargs: здесь можно указать парметры запуска задачи
                     -  params=List[Dict(key='ключ_1', value='значение_1'),..., Dict(key='ключ_N', value='значение_N')];
                   если эти данные можно переиспользовать в автотестах
                     -  auto_test=Dict[alias: str, parallel_use: bool]
        """
        test_data = self.client_provider('CREATE_TASK', case_id=case_id, is_client=False, timeout=timeout, **kwargs)
        attach('Тестовые данные', json.dumps(test_data, indent=4, ensure_ascii=False))
        return test_data

    @step('Освободить клиента после теста')
    def release_client(self):
        """
        Отправить запрос в хранилище тестовых пользователей на освобождение клиента
        :return: MobileContext
        """
        self.client_provider('RELEASE')
        del self.client
        return self

    @step('Запустить подготовку клиента после теста')
    def re_create_client(self):
        """
        Отправить запрос в хранилище тестовых пользователей на запуск подготовки такого же клиента
        :return: MobileContext
        """
        if self.client:
            self.client_provider('RERUN')
        return self

    @step('Удалить клиента после теста')
    def delete_client(self):
        """
        Отправить запрос в хранилище тестовых пользователей - пометить клиента удаленным
        :return: MobileContext
        """
        if self.client:
            self.client_provider('DELETE')
            del self.client
        return self
